@@ -4,7 +4,7 @@
 #'
 #' @export
 #'
-methods::setClass(
+setClass(
    "ClickHouseHTTPConnection",
    contains="DBIConnection",
    slots=list(
@@ -14,15 +14,17 @@ methods::setClass(
       password="function",
       https="logical",
       ssl_verifypeer="logical",
+      host_path="character",
       session="character",
-      convert_uint="logical"
+      convert_uint="logical",
+      extended_headers="list"
    )
 )
 
 ###############################################################################@
 ## dbIsValid ----
 ##
-methods::setMethod(
+setMethod(
    "dbIsValid", "ClickHouseHTTPConnection",
    function(dbObj, ...){
       toRet <- try(.check_db_session(dbObj), silent=TRUE)
@@ -42,7 +44,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbDisconnect ----
 ##
-methods::setMethod(
+setMethod(
    "dbDisconnect", "ClickHouseHTTPConnection",
    function(conn, ...){
       xn <- deparse(substitute(conn))
@@ -105,7 +107,7 @@ methods::setMethod(
 #'
 #' @export
 #'
-methods::setMethod(
+setMethod(
    "dbSendQuery",
    c("ClickHouseHTTPConnection", "character"),
    function(
@@ -146,7 +148,7 @@ methods::setMethod(
          as.numeric
       )
       resEnv$fetched <- FALSE
-      return(methods::new(
+      return(new(
          "ClickHouseHTTPResult",
          sql=statement,
          env=resEnv,
@@ -158,7 +160,7 @@ methods::setMethod(
 ###############################################################################@
 ## show ----
 ##
-methods::setMethod("show", "ClickHouseHTTPConnection", function(object){
+setMethod("show", "ClickHouseHTTPConnection", function(object){
    cat("<ClickHouseHTTPConnection>\n")
    if(DBI::dbIsValid(object)){
       cat(sprintf(
@@ -191,7 +193,7 @@ methods::setMethod("show", "ClickHouseHTTPConnection", function(object){
 #'
 #' @export
 #'
-methods::setMethod(
+setMethod(
    "dbGetInfo", "ClickHouseHTTPConnection",
    function(dbObj, ...){
       chinfo <- DBI::dbGetQuery(
@@ -217,7 +219,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbListTables ----
 ##
-methods::setMethod(
+setMethod(
    "dbListTables", "ClickHouseHTTPConnection",
    function(conn, ...){
       return(as.character(DBI::dbGetQuery(conn, "SHOW TABLES")[[1]]))
@@ -227,7 +229,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbDataType ----
 ##
-methods::setMethod(
+setMethod(
    "dbDataType", "ClickHouseHTTPConnection", function(dbObj, obj, ...){
       return(DBI::dbDataType(ClickHouseHTTP(), obj))
    },
@@ -237,7 +239,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbQuoteIdentifier.character ----
 ##
-methods::setMethod(
+setMethod(
    "dbQuoteIdentifier", c("ClickHouseHTTPConnection", "character"),
     function(conn, x, ...){
        if(anyNA(x)){
@@ -253,7 +255,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbQuoteIdentifier.SQL ----
 ##
-methods::setMethod(
+setMethod(
    "dbQuoteIdentifier", c("ClickHouseHTTPConnection", "SQL"),
    function(conn, x, ...){
       return(x)
@@ -263,7 +265,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbQuoteString.character ----
 ##
-methods::setMethod(
+setMethod(
    "dbQuoteString", c("ClickHouseHTTPConnection", "character"),
    function(conn, x, ...){
       x <- gsub('\\', '\\\\', x, fixed=TRUE)
@@ -275,7 +277,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbQuoteString.SQL ----
 ##
-methods::setMethod(
+setMethod(
    "dbQuoteString", c("ClickHouseHTTPConnection", "SQL"),
    function(conn, x, ...){
       return(x)
@@ -285,7 +287,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbExistsTable ----
 ##
-methods::setMethod(
+setMethod(
    "dbExistsTable", c("ClickHouseHTTPConnection", "character"),
    function(conn, name, ...){
       qname <- DBI::dbQuoteIdentifier(conn, name)
@@ -296,7 +298,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbReadTable ----
 ##
-methods::setMethod(
+setMethod(
    "dbReadTable", c("ClickHouseHTTPConnection", "character"),
    function(conn, name,...){
       qname <- DBI::dbQuoteIdentifier(conn, name)
@@ -332,7 +334,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbListFields ----
 ##
-methods::setMethod(
+setMethod(
    "dbListFields", c("ClickHouseHTTPConnection", "character"),
    function(conn, name, ...){
    qname <- DBI::dbQuoteIdentifier(conn, name)
@@ -342,7 +344,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbRemoveTable ----
 ##
-methods::setMethod(
+setMethod(
    "dbRemoveTable", "ClickHouseHTTPConnection",
    function(conn, name, ...){
       qname <- DBI::dbQuoteIdentifier(conn, name)
@@ -382,7 +384,7 @@ methods::setMethod(
 #'
 #' @export
 #'
-methods::setMethod(
+setMethod(
    "dbCreateTable", "ClickHouseHTTPConnection",
    function(
       conn, name, fields, engine="TinyLog",
@@ -410,7 +412,7 @@ methods::setMethod(
 ###############################################################################@
 ## dbAppendTable ----
 ##
-methods::setMethod(
+setMethod(
    "dbAppendTable", "ClickHouseHTTPConnection",
    function(
       conn, name, value, ..., row.names=NULL
@@ -452,7 +454,7 @@ methods::setMethod(
 #'
 #' @export
 #'
-methods::setMethod(
+setMethod(
    "dbWriteTable", "ClickHouseHTTPConnection",
    function(
       conn, name, value,
@@ -489,14 +491,16 @@ methods::setMethod(
 ## Helpers ----
 
 .build_http_req <- function(
-   host, port, https,
+   host, port, https, host_path,
    session, session_timeout=NA,
    query=""
 ){
    sprintf(
-      "%s://%s:%s/?session_id=%s&session_check=%s%s&query=%s",
+      "%s://%s:%s/%s?session_id=%s&session_check=%s%s&query=%s",
       ifelse(https, "https", "http"),
-      host, port, session, as.integer(is.na(session_timeout)),
+      host, port,
+      ifelse(is.na(host_path), "", paste0(host_path, "/")),
+      session, as.integer(is.na(session_timeout)),
       ifelse(
          is.na(session_timeout), "",
          sprintf("&session_timeout=%s", as.integer(session_timeout))
@@ -518,17 +522,24 @@ methods::setMethod(
       qbody <- httr::upload_file(file)
       query <- utils::URLencode(query)
    }
+
+   ## Add authentication headers ----
+   qheaders <- list(
+      'X-ClickHouse-User'=dbc@user,
+      'X-ClickHouse-Key'=dbc@password()
+   )
+   ## Add other headers ----
+   qheaders <- c(qheaders, dbc@extended_headers)
+   
    httr::POST(
       url=.build_http_req(
          host=dbc@host, port=dbc@port, https=dbc@https,
+         host_path=dbc@host_path,
          session=dbc@session, session_timeout=session_timeout,
          query=query
       ),
       body=qbody,
-      httr::add_headers(
-         'X-ClickHouse-User'=dbc@user,
-         'X-ClickHouse-Key'=dbc@password()
-      ),
+      do.call(add_headers, qheaders),
       config=httr::config(ssl_verifypeer=as.integer(dbc@ssl_verifypeer))
    )
 }
@@ -557,6 +568,13 @@ methods::setMethod(
 .check_db_session <- function(
    dbc, session_timeout=NA
 ){
-   r <- .send_query(dbc=dbc, query="SELECT 1", session_timeout=session_timeout)
-   return(.query_success(r))
+   r <- .send_query(
+      dbc=dbc, query="SELECT currentUser() as user",
+      session_timeout=session_timeout
+   )
+   toRet <- .query_success(r)
+   if(toRet){
+      attr(toRet, "user") <- sub("\n$", "", rawToChar(r$content))
+   }
+   return(toRet)
 }
